import { NgIf, AsyncPipe, NgTemplateOutlet, formatDate } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, inject, TemplateRef, Directive, Input, Component, ViewEncapsulation, ChangeDetectionStrategy, ContentChildren, NgModule, LOCALE_ID } from '@angular/core';
import { MAT_FORM_FIELD } from '@angular/material/form-field';
import { defer, of } from 'rxjs';
import { startWith, map, distinctUntilChanged } from 'rxjs/operators';

/** @docs-private */
function getNgxMatErrorDefMissingForError() {
    return Error(`'for' must be set for ngxMatErrorDef. See example: *ngxMatErrorDef="let error; for: 'pattern'`);
}

const NGX_MAT_ERROR_DEFAULT_OPTIONS = new InjectionToken('NGX_MAT_ERROR_DEFAULT_OPTIONS');
class NgxMatErrorDef {
    constructor() {
        this.template = inject(TemplateRef);
    }
    ngOnInit() {
        if (!this.ngxMatErrorDefFor) {
            throw getNgxMatErrorDefMissingForError();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorDef, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrorDef, isStandalone: true, selector: "[ngxMatErrorDef]", inputs: { ngxMatErrorDefFor: "ngxMatErrorDefFor" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorDef, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxMatErrorDef]',
                    standalone: true,
                }]
        }], propDecorators: { ngxMatErrorDefFor: [{
                type: Input
            }] } });
class NgxMatErrors {
    constructor() {
        this.messages = inject(NGX_MAT_ERROR_DEFAULT_OPTIONS);
        this.matFormField = inject(MAT_FORM_FIELD, { optional: true });
        this.error$ = defer(() => {
            const control = (this.control || this.matFormField?._control)?.ngControl
                ?.control;
            if (!control) {
                // TODO: Throw error;
                return of(undefined);
            }
            return control.valueChanges.pipe(startWith(null), map(() => {
                if (!control.errors) {
                    return;
                }
                const errorKeys = Object.keys(control.errors);
                const errorOrErrorDef = this.customErrorMessages.find((customErrorMessage) => errorKeys.some((error) => error === customErrorMessage.ngxMatErrorDefFor)) ?? errorKeys.find((key) => key in this.messages);
                if (!errorOrErrorDef) {
                    return;
                }
                const errors = control.errors;
                if (errorOrErrorDef instanceof NgxMatErrorDef) {
                    return {
                        template: errorOrErrorDef.template,
                        $implicit: errors[errorOrErrorDef.ngxMatErrorDefFor],
                    };
                }
                const message = this.messages[errorOrErrorDef];
                return {
                    $implicit: typeof message === 'function'
                        ? message(errors[errorOrErrorDef])
                        : message,
                };
            }), 
            // this distincts only undefined values
            distinctUntilChanged());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrors, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrors, isStandalone: true, selector: "ngx-mat-errors, [ngx-mat-errors]", inputs: { control: ["ngx-mat-errors", "control"] }, host: { classAttribute: "ngx-mat-errors" }, queries: [{ propertyName: "customErrorMessages", predicate: NgxMatErrorDef, descendants: true }], ngImport: i0, template: `<ng-template #defaultTemplate let-error>{{ error }}</ng-template
    ><ng-template [ngIf]="error$ | async" let-error
      ><ng-template
        [ngTemplateOutlet]="error.template ?? defaultTemplate"
        [ngTemplateOutletContext]="error"
      ></ng-template>
    </ng-template>`, isInline: true, dependencies: [{ kind: "directive", type: NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: AsyncPipe, name: "async" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrors, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngx-mat-errors, [ngx-mat-errors]',
                    template: `<ng-template #defaultTemplate let-error>{{ error }}</ng-template
    ><ng-template [ngIf]="error$ | async" let-error
      ><ng-template
        [ngTemplateOutlet]="error.template ?? defaultTemplate"
        [ngTemplateOutletContext]="error"
      ></ng-template>
    </ng-template>`,
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    standalone: true,
                    imports: [NgIf, AsyncPipe, NgTemplateOutlet],
                    host: {
                        class: 'ngx-mat-errors',
                    },
                }]
        }], propDecorators: { customErrorMessages: [{
                type: ContentChildren,
                args: [NgxMatErrorDef, { descendants: true }]
            }], control: [{
                type: Input,
                args: ['ngx-mat-errors']
            }] } });

class NgxMatErrorsModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, imports: [NgxMatErrors, NgxMatErrorDef], exports: [NgxMatErrors, NgxMatErrorDef] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [NgxMatErrors, NgxMatErrorDef],
                    exports: [NgxMatErrors, NgxMatErrorDef],
                }]
        }] });

function errorMessagesEnFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Please enter a value greater than or equal to ${error.min}.`,
        max: (error) => `Please enter a value less than or equal to ${error.max}.`,
        required: `This field is required.`,
        email: `Please enter a valid email address.`,
        minlength: (error) => `Please enter at least ${error.requiredLength} characters.`,
        maxlength: (error) => `Please enter no more than ${error.requiredLength} characters.`,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            return `Please enter a date greater than or equal to ${formatted ?? error.min}.`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            return `Please enter a date less than or equal to ${formatted ?? error.max}.`;
        },
    };
}
const NGX_MAT_ERROR_CONFIG_EN = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesEnFactory,
    deps: [LOCALE_ID],
};

function errorMessagesHuFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Nem lehet kisebb, mint ${error.min}.`,
        max: (error) => `Nem lehet nagyobb, mint ${error.max}.`,
        required: `Kötelező mező.`,
        email: `Nem érvényes e-mail cím.`,
        minlength: (error) => `Legalább ${error.requiredLength} karakter hosszú lehet.`,
        maxlength: (error) => `Legfeljebb ${error.requiredLength} karakter hosszú lehet.`,
        server: (error) => error,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            // In Hungarian date ends with '.'
            return `Nem lehet korábbi dátum, mint ${formatted ?? error.min}`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            // In Hungarian date ends with '.'
            return `Nem lehet későbbi dátum, mint ${formatted ?? error.max}`;
        },
    };
}
const NGX_MAT_ERROR_CONFIG_HU = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesHuFactory,
    deps: [LOCALE_ID]
};

function errorMessagesPtBtFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Informe um valor igual ou maior a ${error.min}.`,
        max: (error) => `Informe um valor igual ou menor a ${error.max}.`,
        required: `Campo obrigatório.`,
        email: `Informe um endereço de email válido.`,
        minlength: (error) => `Informe pelo menos ${error.requiredLength} caracteres.`,
        maxlength: (error) => `O campo não pode ter mais que ${error.requiredLength} caracteres.`,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            return `Informe uma data maior ou igual a ${formatted ?? error.min}.`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            return `Informe uma data menor ou igual a ${formatted ?? error.max}.`;
        },
    };
}
const NGX_MAT_ERROR_CONFIG_PT_BR = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesPtBtFactory,
    deps: [LOCALE_ID],
};

/*
 * Public API Surface of ngx-mat-errors
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGX_MAT_ERROR_CONFIG_EN, NGX_MAT_ERROR_CONFIG_HU, NGX_MAT_ERROR_CONFIG_PT_BR, NGX_MAT_ERROR_DEFAULT_OPTIONS, NgxMatErrorDef, NgxMatErrors, NgxMatErrorsModule, errorMessagesEnFactory, errorMessagesHuFactory, errorMessagesPtBtFactory };
//# sourceMappingURL=ngx-mat-errors.mjs.map
